"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TypesManager = void 0;

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _lodash = require("lodash");

var _util = require("util");

var _apolloServer = require("apollo-server");

var _suryaGqlScalar = require("surya-gql-scalar");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const manifestName = "Manifest";
const queriesSuffix = "Queries";
const resolversSuffux = "Resolvers";
const managerSuffix = "Manager";
const emptyQuery = _apolloServer.gql`
  type Query {
    _empty: String
  }

  type Mutation {
    _empty: String
  }
`;
const emptyResolvers = {
  Query: {}
};

class TypesManager {
  constructor(container) {
    this.container = container;
    this.log = container.log;
    this.dataInterfaces = container.dataInterfaces;
    this.types = {};
    this.schemas = [];
    this.queries = [emptyQuery, _suryaGqlScalar.Scalars];
    this.resolvers = (0, _lodash.merge)(emptyResolvers, _suryaGqlScalar.DateScalar);
    this.managers = {};
    this.interfaceMap = {};
  }

  async loadTypeModules(basePath) {
    return new Promise((resolve, reject) => {
      _fs.default.readdir(basePath, (err, items) => {
        if (err) {
          reject(err);
        } else {
          this.log.info(`LoadTypeModules: ${items}`);
          items.forEach(item => {
            const itemPath = _path.default.join(basePath, item);

            const isDir = _fs.default.lstatSync(itemPath).isDirectory();

            this.log.info(`--------------------------------\n>>Item: ${itemPath} isDir: ${isDir}`);

            if (isDir) {
              try {
                this.log.info(`require(${itemPath})`);

                const typeModule = require(itemPath);

                let typeModuleItems = Object.keys(typeModule);

                if (typeModuleItems.includes(manifestName)) {
                  const manifest = typeModule.Manifest;
                  typeModuleItems = this.arrayRemove(typeModuleItems, manifestName);
                  const types = manifest.types || [];
                  let baseTypeName = this.firstUpperCase(item);

                  if (types.length > 0) {
                    baseTypeName = types[0];
                  } //load Queries


                  this.log.info(`LoadQueries...`);
                  typeModuleItems = this.loadQueries(baseTypeName, manifest, typeModule, typeModuleItems); // load Resolvers

                  this.log.info(`LoadResolvers...`);
                  typeModuleItems = this.loadResolvers(baseTypeName, manifest, typeModule, typeModuleItems); // load Manager

                  this.log.info(`LoadManager...`);

                  try {
                    typeModuleItems = this.loadManager(baseTypeName, manifest, typeModule, typeModuleItems);
                  } catch (ex) {
                    this.log.error(`Error loading Manager ${ex.message}`);
                    reject(ex);
                  } // capture API info


                  this.log.info(`LoadAPI...`);
                  typeModuleItems = this.loadAPI(baseTypeName, manifest, typeModule, typeModuleItems); // load all Types

                  this.log.info(`LoadTypes...`);

                  try {
                    this.loadTypes(baseTypeName, manifest, typeModule, typeModuleItems);
                  } catch (ex) {
                    this.log.error(`Error loading types: ${ex.message}`);
                    reject(ex);
                  }
                } else {
                  this.log.warn(`No Manifest: "${item}" Is not a legit type.`);
                }
              } catch (ex) {
                this.log.warn(`${item}: ${ex}`);
              }
            }
          });

          try {
            this.createAPIs();
          } catch (ex) {
            this.log.error(`Error creating APIs ${ex.message}`);
            reject(ex);
          }
        }

        resolve(true);
      });
    });
  }

  loadQueries(baseTypeName, manifest, typeModule, typeModuleItems) {
    let queriesName = manifest.queries || baseTypeName + queriesSuffix;
    typeModuleItems = this.arrayRemove(typeModuleItems, queriesName);
    const queries = typeModule[queriesName] || null;

    if (queries) {
      this.queries = this.queries.concat(queries);
    } else {
      this.log.warn(`No Queries Found for ${baseTypeName}`);
    }

    return typeModuleItems;
  }

  loadResolvers(baseTypeName, manifest, typeModule, typeModuleItems) {
    let resolversName = manifest.resolvers || baseTypeName + resolversSuffux;
    typeModuleItems = this.arrayRemove(typeModuleItems, resolversName);
    const resolvers = typeModule[resolversName] || null;

    if (resolvers) {
      this.resolvers = (0, _lodash.merge)(this.resolvers, resolvers);
    } else {
      this.log.warn(`No Resolvers Found for ${baseTypeName}`);
    }

    return typeModuleItems;
  }

  loadManager(baseTypeName, manifest, typeModule, typeModuleItems) {
    const typeClass = typeModule[baseTypeName] || null;

    if (!typeClass) {
      throw new Error(`No Type found for : ${baseTypeName}`);
    }

    let managerName = manifest.manager || baseTypeName + managerSuffix;
    typeModuleItems = this.arrayRemove(typeModuleItems, managerName);
    this.log.info(`LoadManger: ${managerName}`);
    const managerClass = typeModule[managerName] || null;

    if (managerClass) {
      const manager = new managerClass(baseTypeName, typeClass, this.container);
      this.managers[baseTypeName] = manager;
    } else {
      this.log.warn(`No Manager found for ${baseTypeName}`);
    }

    return typeModuleItems;
  }

  loadAPI(baseTypeName, manifest, typeModule, typeModuleItems) {
    let dataConfig = manifest.data || null;

    if (dataConfig) {
      const interfaceName = dataConfig.interface;
      const apiName = dataConfig.api;
      const api = typeModule[apiName] || null;

      if (api) {
        typeModuleItems = this.arrayRemove(typeModuleItems, apiName);
        const apiTypeName = dataConfig.type || baseTypeName;
        const apiType = typeModule[apiTypeName] || null;

        if (!apiType) {
          this.log.error(`API base type not found: ${apiTypeName}`);
        } // save api and typeName in per-interface map list for instantiation after types are loaded.


        this.log.info(`Capture API info: interface: ${interfaceName} api: ${apiName} isFunc: ${(0, _util.isFunction)(api)}`);
        let interfaceEntry = this.interfaceMap[interfaceName] || {
          types: [],
          apis: []
        };
        interfaceEntry.types.push(apiType);
        interfaceEntry.apis.push({
          apiTypeName,
          apiName,
          api
        });
        this.interfaceMap[interfaceName] = interfaceEntry;
      } else {
        this.log.error(`API not found: ${apiName}`);
      }
    } else {
      this.log.info(`No data interface config for: ${baseTypeName}`);
    }

    return typeModuleItems;
  }
  /**
   * Create all APIs with data collected.
   */


  createAPIs() {
    this.log.info(`Create APIs: ${JSON.stringify(this.interfaceMap, null, 2)}`);
    Object.keys(this.interfaceMap).forEach(iname => {
      this.log.info(`Interface: ${iname}`);
      const ifdata = this.interfaceMap[iname];
      const dataInterface = this.dataInterfaces[iname]; // prep all of the base types associated with all apis using this interface
      // This allows implementation of joins across Types within the same interface system

      dataInterface.loadTypes(ifdata.types); // create and register each API using this interface.

      ifdata.apis.forEach(api => {
        const apiTypeName = api.apiTypeName;
        const apiClass = api.api;
        const apiTypeClass = this.types[apiTypeName];

        if ((0, _util.isFunction)(apiClass)) {
          try {
            const apiObj = dataInterface.loadAPI(apiTypeName, apiClass, apiTypeClass);
            this.managers[apiTypeName].setAPI(apiObj);
            this.log.info(`API ${api.apiName} created!`);
          } catch (ex) {
            this.log.error(`Error loading API: ${api.apiName} ${ex}`);
          }
        } else {
          this.log.warn(`API is not a class: ${api.apiName}`);
        }
      });
    });
  }
  /**
   * If the manifest declares the types, use that list.
   * If it does not, the list in typeModuleItems should contain whatever is left over
   * after all other items are loaded and redacted from the list.
   * As such it should tolerate erroneous entries with warnings.
   * @param {*} baseTypeName
   * @param {*} manifest
   * @param {*} typeModule
   * @param {*} typeModuleItems
   */


  loadTypes(baseTypeName, manifest, typeModule, typeModuleItems) {
    const loadTypeNames = manifest.types || typeModuleItems;
    this.log.info(`Load Types for  ${baseTypeName}: ${loadTypeNames}`);
    loadTypeNames.forEach(typeName => {
      this.loadType(typeName, typeModule);
    });
  }

  loadType(typeName, typeModule) {
    const typeClass = typeModule[typeName] || null;

    if (typeClass && (0, _util.isFunction)(typeClass)) {
      this.log.info(`LoadType: ${typeName}`);

      try {
        const typeObj = new typeClass();
        this.types[typeName] = typeClass; // save the Class, not the obj in types

        const schema = typeObj.toGraphQLSchema();
        this.schemas = this.schemas.concat(schema);
      } catch (ex) {
        this.log.warn(`Error loading Txype: ${ex}`);
      }
    } else {
      this.log.error(`loadType not a function: ${typeName}`);
    }
  }

  arrayRemove(arr, itemName) {
    const ix = arr.indexOf(itemName);

    if (ix >= 0) {
      arr = (0, _lodash.concat)(arr.slice(0, ix), arr.slice(ix + 1));
    }

    return arr;
  }

  firstUpperCase(str) {
    if (typeof str === "string") {
      return str[0].toUpperCase() + str.slice(1);
    }

    return null;
  }

  getTypeDefs() {
    return this.queries.concat(this.schemas);
  }

  getResolvers() {
    return this.resolvers;
  }

  getManagers() {
    return this.managers;
  }

}

exports.TypesManager = TypesManager;
//# sourceMappingURL=typesManager.js.map