import { AuthenticationError, ForbiddenError } from "apollo-server"

const authorized = {
  get: { groups: ["admin"] },
  query: { groups: ["admin"] },
  create: { groups: ["admin"] },
  update: { groups: ["admin"] },
  delete: { groups: ["admin"] },
}

export class BaseManager {
  constructor(typeName, type, container) {
    this.typeName = typeName
    this.type = type
    this.container = container
    this.log = container.log
    this.session = container.session
    this.api = null
  }

  authorized() {
    return authorized
  }

  getNewInstance() {
    return new this.type()
  }

  setAPI(api) {
    this.api = api
    this.log.info(`setAPI for Manager for ${this.typeName}`)
  }

  getAPI() {
    return this.api
  }

  // ===================================================
  // perform authentication and authorization

  async authorize(action, doAuthorize, obj, auth) {
    if (!doAuthorize) return
    const userInfo = await this.session.get(auth.token)
    if (!userInfo) {
      throw new AuthenticationError("User session is not authenticated")
    }

    const actionAuth = this.authorized()[action] || {}

    const checkOwner = actionAuth.owner || false
    let isOwner = true
    if (checkOwner) {
      const authUserId = userInfo.user._id || ""
      isOwner = obj.isOwner(authUserId)
    }

    const isInGroups = this.userInGroups(actionAuth.groups, userInfo)

    if (!(isInGroups || (checkOwner && isOwner))) {
      throw new ForbiddenError("User is not authorized ")
    }
  }

  async applyReadPermissions(doAuthorize, obj, auth) {
    if (!doAuthorize) {
      return obj
    }

    let userInfo = await this.session.get(auth.token)
    if (!userInfo) {
      throw new AuthenticationError("User session is not authenticated")
    }

    const authUserId = userInfo.user._id || ""
    const secureGroups = obj.getPermissions("Secure")
    const typeName = obj.constructor.name

    let isInSecureGroups = this.userInGroups(secureGroups, userInfo)
    if (isInSecureGroups) {
      return obj
    } else {
      return obj.applyReadPermissions(authUserId)
    }
  }

  async applyQueryReadPermissions(doAuthorize, items, auth) {
    if (!doAuthorize) return items

    let userInfo = await this.session.get(auth.token)
    if (!userInfo) {
      throw new AuthenticationError("User session is not authenticated")
    }

    const authUserId = userInfo.user._id || ""
    const obj = this.getNewInstance()
    const secureGroups = obj.getPermissions("Secure")

    const isInSecureGroups = this.userInGroups(secureGroups, userInfo)

    if (!isInSecureGroups) {
      let filtered = items.map(obj => {
        return obj.applyReadPermissions(authUserId)
      })
      return filtered
    } else {
      return items
    }
  }

  /**
   * Allow for future softening of user->group membership system
   */
  userInGroups(groupList, userInfo) {
    for (let group of groupList) {
      switch (group) {
        case "*":
          return true
          break
        case "admin":
          if (userInfo.user.administrator) {
            return true
            break
          }
      }
    }
    return false
  }
}
